<?php
/**
 * @file
 * Theme file for Projekktor
 */

/**
 * Projekktor template preprocess function.
 */
function template_preprocess_projekktor_container(&$vars) {
  $settings = &$vars['settings'];

  // Each Projekktor instance gets a unique id.
  $projekktor_id = &drupal_static('projekktor_id', 0);
  $vars['id'] = ++$projekktor_id;

  // Prepare elements.
  $items = $vars['items'];
  $vars['items'] = array();
  foreach ($items as $delta => $item) {
    $playlist[] = array();

    // Gets our title if we have one.
    if (isset($item['filename'])) {
      $title = $item['filename'];
    }
    else $title = NULL;

    // Creates a playlist item.
    $playlist_item = array(
      $item['key'] = array(
        'src' => file_create_url($item['uri']),
        'type' => $item['filemime'],
      ),
      'config' => array(
        'title' => $title, 
      ),
    );

    $playlist[$delta] = $playlist_item;
  }

  // Convert $id to human friendly form for projekktor_add API.
  $player_id = 'projekktor-' . $vars['id'];
  // Convert $settings to human friendly form for projekktor_add API.
  $preset_name = $settings['projekktor_preset'];
  $theme_name = $settings['projekktor_theme'];

  // Attach Projekktor JavaScript.
  projekktor_add($player_id, $preset_name, $theme_name, $playlist);
}

/**
 * Views template preprocess function.
 */
function theme_views_view_projekktor($vars) {
  $items = array();

  $view = $vars['view'];
  $file_field_name = $vars['file_field_name'];
  foreach ($vars['rows'] as $row) {
    $lang = $row->_field_data[$view->base_field]['entity']->language;
    // omit rows without file field.
    if (!isset($row->_field_data[$view->base_field]['entity']->{$file_field_name})) {
      continue;
    }
    $item = $row->_field_data[$view->base_field]['entity']->{$file_field_name}[$lang][0];
    $items[] = $item;
  }

  return theme('projekktor_container', array(
    'items' => $items,
    'settings' => $vars['options'],
  ));
}
